function [x_0, newmask] = GetNewStartingPoint(im, grad, mask, param)
    % x_0 is a 2x1 vector (x,y), pointing to a pixel of im, indicating a 
    % relevant location for starting to draw a new line.
    % 
    % Throws a 'GRADLINES:GetNewStartingPoint:NoNewStartingPoint'
    % exception if no relevant point can be found.
    % 
    % param :
    %       param.mask.start.type = {'multiplicative', 'additive'}
    %           N.B. : binary is a sub-type of multiplicative
    %       param.mask.start.method = {'random', 'left-top'}
    %           Indicates which point we should chose among the argmax of
    %           mask.start .
    %       param.mask.start.filter : local filter matrix, applied to
    %           mask.start centered around x_0 
    %       param.mask.start.thresh : double
    %           N.B. : typically, between 0 and 1
    %
    % Author : Jean Feydy
    
    newmask = mask;
    val = unique(max(mask.start(:)));
    % I et J contiennent les abscisses et ordonnees des points de depart possibles
    [I,J] = find(mask.start==val);
    assert(isequal(size(I), size(J)));
    
    n_choix = numel(I);
    if isequal(n_choix, 0)
        throw(MException('GRADLINES:GetNewStartingPoint:NoNewStartingPoint', 'No new starting point found'));
    end;
    indice = 0;
    if strcmp(param.mask.start.method, 'left-top')
        indice = 1;
    elseif strcmp(param.mask.start.method, 'random')
        indice = randi(n_choix);
    else
        error('param.mask.starting.method should be either left-top or random');
    end;
    
    x_0 = [J(indice); I(indice)];
    ApplyStartMaskOnNonPaddedArray(x_0);
    x_0 = x_0 - [0.5; 0.5]; % A pixel is a 1x1 square, we want to start at its center
    
    function ApplyStartMaskOnNonPaddedArray(x)
        [sx, sy] = size(param.mask.start.filter);
        assert(isequal(sx, sy), 'Start filter must be a square matrix.');
        assert(isequal(mod(sx, 2), 1), 'Start filter must be (odd x odd) (--> centered).');
        mask_padding = (sx - 1)/2;
        padded_mask = padarray(mask.start, [mask_padding, mask_padding]);

        % We apply the start filter around the point x
        if isequal(param.mask.start.type, 'multiplicative')
            padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) = ...
                padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) .* param.mask.start.filter;
        elseif isequal(param.mask.start.type, 'additive')
            padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) = ...
                padded_mask(x(2):x(2)+ 2*mask_padding, x(1):x(1)+ 2*mask_padding) + param.mask.start.filter;
        else
            error('param.mask.start.type must be either multiplicative or additive');
        end;
        newmask.start = padded_mask(mask_padding + 1:end-mask_padding, mask_padding + 1:end-mask_padding);
    end
end